@@ -28,12 +28,15 @@ import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 import java.util.regex.Pattern;
 
-import javax.servlet.http.HttpServletResponse;
+import jakarta.servlet.ServletConnection;
+import jakarta.servlet.http.HttpServletResponse;
 
 import org.apache.coyote.AbstractProcessor;
 import org.apache.coyote.ActionCode;
@@ -86,7 +89,7 @@ public class AjpProcessor extends AbstractProcessor {
     private static final byte[] pongMessageArray;
 
 
-    private static final Set<String> javaxAttributes;
+    private static final Map<String, String> jakartaAttributeMapping;
     private static final Set<String> iisTlsAttributes;
 
 
@@ -127,13 +130,17 @@ public class AjpProcessor extends AbstractProcessor {
         pongMessageArray = new byte[pongMessage.getLen()];
         System.arraycopy(pongMessage.getBuffer(), 0, pongMessageArray, 0, pongMessage.getLen());
 
-        // Build the Set of javax attributes
-        Set<String> s2 = new HashSet<>();
-        s2.add("javax.servlet.request.cipher_suite");
-        s2.add("javax.servlet.request.key_size");
-        s2.add("javax.servlet.request.ssl_session");
-        s2.add("javax.servlet.request.X509Certificate");
-        javaxAttributes = Collections.unmodifiableSet(s2);
+        // Build Map of Java Servlet to Jakarta Servlet attribute names
+        Map<String, String> m = new HashMap<>();
+        m.put("jakarta.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite");
+        m.put("jakarta.servlet.request.key_size", "jakarta.servlet.request.key_size");
+        m.put("jakarta.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session");
+        m.put("jakarta.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
+        m.put("javax.servlet.request.cipher_suite", "jakarta.servlet.request.cipher_suite");
+        m.put("javax.servlet.request.key_size", "jakarta.servlet.request.key_size");
+        m.put("javax.servlet.request.ssl_session", "jakarta.servlet.request.ssl_session");
+        m.put("javax.servlet.request.X509Certificate", "jakarta.servlet.request.X509Certificate");
+        jakartaAttributeMapping = Collections.unmodifiableMap(m);
 
         Set<String> s = new HashSet<>();
         s.add("CERT_ISSUER");
@@ -367,11 +374,11 @@ public class AjpProcessor extends AbstractProcessor {
                     try {
                         socketWrapper.write(true, pongMessageArray, 0, pongMessageArray.length);
                         socketWrapper.flush(true);
-                    } catch (IOException ioe) {
+                    } catch (IOException e) {
                         if (getLog().isDebugEnabled()) {
-                            getLog().debug(sm.getString("ajpprocessor.pongFail"), ioe);
+                            getLog().debug(sm.getString("ajpprocessor.pongFail"), e);
                         }
-                        setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
+                        setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                     }
                     recycle();
                     continue;
@@ -384,15 +391,13 @@ public class AjpProcessor extends AbstractProcessor {
                     setErrorState(ErrorState.CLOSE_CONNECTION_NOW, null);
                     break;
                 }
-                request.setStartTime(System.currentTimeMillis());
-            } catch (IOException ioe) {
-                setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
+                request.setStartTimeNanos(System.nanoTime());
+            } catch (IOException e) {
+                setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                 break;
             } catch (Throwable t) {
                 ExceptionUtils.handleThrowable(t);
-                if (getLog().isDebugEnabled()) {
-                    getLog().debug(sm.getString("ajpprocessor.header.error"), t);
-                }
+                getLog().debug(sm.getString("ajpprocessor.header.error"), t);
                 // 400 - Bad Request
                 response.setStatus(400);
                 setErrorState(ErrorState.CLOSE_CLEAN, t);
@@ -405,9 +410,7 @@ public class AjpProcessor extends AbstractProcessor {
                     prepareRequest();
                 } catch (Throwable t) {
                     ExceptionUtils.handleThrowable(t);
-                    if (getLog().isDebugEnabled()) {
-                        getLog().debug(sm.getString("ajpprocessor.request.prepare"), t);
-                    }
+                    getLog().debug(sm.getString("ajpprocessor.request.prepare"), t);
                     // 500 - Internal Server Error
                     response.setStatus(500);
                     setErrorState(ErrorState.CLOSE_CLEAN, t);
@@ -675,7 +678,7 @@ public class AjpProcessor extends AbstractProcessor {
         boolean contentLengthSet = false;
         int hCount = requestHeaderMessage.getInt();
         for (int i = 0; i < hCount; i++) {
-            String hName;
+            String hName = null;
 
             // Header names are encoded as either an integer code starting
             // with 0xA0, or as a normal string (in which case the first
@@ -683,7 +686,7 @@ public class AjpProcessor extends AbstractProcessor {
             int isc = requestHeaderMessage.peekInt();
             int hId = isc & 0xFF;
 
-            MessageBytes vMB;
+            MessageBytes vMB = null;
             isc &= 0xFF00;
             if (0xA000 == isc) {
                 requestHeaderMessage.getInt(); // To advance the read position
@@ -717,7 +720,7 @@ public class AjpProcessor extends AbstractProcessor {
             } else if (hId == Constants.SC_REQ_CONTENT_TYPE || (hId == -1 && tmpMB.equalsIgnoreCase("Content-Type"))) {
                 // just read the content-type header, so set it
                 ByteChunk bchunk = vMB.getByteChunk();
-                request.contentType().setBytes(bchunk.getBytes(), bchunk.getStart(), bchunk.getLength());
+                request.contentType().setBytes(bchunk.getBytes(), bchunk.getOffset(), bchunk.getLength());
             }
         }
 
@@ -751,8 +754,10 @@ public class AjpProcessor extends AbstractProcessor {
                         request.setAttribute(SSLSupport.PROTOCOL_VERSION_KEY, v);
                     } else if (n.equals("JK_LB_ACTIVATION")) {
                         request.setAttribute(n, v);
-                    } else if (javaxAttributes.contains(n)) {
-                        request.setAttribute(n, v);
+                    } else if (jakartaAttributeMapping.containsKey(n)) {
+                        // AJP uses the Java Servlet attribute names.
+                        // Need to convert these to Jakarta Servlet.
+                        request.setAttribute(jakartaAttributeMapping.get(n), v);
                     } else if (iisTlsAttributes.contains(n)) {
                         // Allow IIS TLS attributes
                         request.setAttribute(n, v);
@@ -836,7 +841,7 @@ public class AjpProcessor extends AbstractProcessor {
 
                 case Constants.SC_A_SECRET:
                     requestHeaderMessage.getBytes(tmpMB);
-                    if (secret != null && !secret.isEmpty()) {
+                    if (secret != null && secret.length() > 0) {
                         secretPresentInRequest = true;
                         if (!tmpMB.equals(secret)) {
                             response.setStatus(403);
@@ -854,7 +859,7 @@ public class AjpProcessor extends AbstractProcessor {
         }
 
         // Check if secret was submitted if required
-        if (secret != null && !secret.isEmpty() && !secretPresentInRequest) {
+        if (secret != null && secret.length() > 0 && !secretPresentInRequest) {
             response.setStatus(403);
             setErrorState(ErrorState.CLOSE_CLEAN, null);
         }
@@ -865,9 +870,10 @@ public class AjpProcessor extends AbstractProcessor {
 
             int pos = uriBC.indexOf("://", 0, 3, 4);
             int uriBCStart = uriBC.getStart();
+            int slashPos = -1;
             if (pos != -1) {
                 byte[] uriB = uriBC.getBytes();
-                int slashPos = uriBC.indexOf('/', pos + 3);
+                slashPos = uriBC.indexOf('/', pos + 3);
                 if (slashPos == -1) {
                     slashPos = uriBC.getLength();
                     // Set URI as "/"
@@ -899,9 +905,9 @@ public class AjpProcessor extends AbstractProcessor {
     protected void populateHost() {
         try {
             request.serverName().duplicate(request.localName());
-        } catch (IOException ioe) {
+        } catch (IOException e) {
             response.setStatus(400);
-            setErrorState(ErrorState.CLOSE_CLEAN, ioe);
+            setErrorState(ErrorState.CLOSE_CLEAN, e);
         }
     }
 
@@ -985,8 +991,8 @@ public class AjpProcessor extends AbstractProcessor {
                     responseMessage.appendBytes(hV);
                 } catch (IllegalArgumentException iae) {
                     // Log the problematic header
-                    log.warn(sm.getString("ajpprocessor.response.invalidHeader", headers.getName(i),
-                            headers.getValue(i)), iae);
+                    log.warn(sm.getString("ajpprocessor.response.invalidHeader", headers.getName(i), headers.getValue(i)),
+                            iae);
                     // Remove the problematic header
                     headers.removeHeader(i);
                     numHeaders--;
@@ -1004,6 +1010,9 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
+    /**
+     * Callback to write data from the buffer.
+     */
     @Override
     protected final void flush() throws IOException {
         // Calling code should ensure that there is no data in the buffers for
@@ -1019,6 +1028,9 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
+    /**
+     * Finish AJP response.
+     */
     @Override
     protected final void finishResponse() throws IOException {
         if (responseFinished) {
@@ -1048,12 +1060,6 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
-    @Override
-    protected void earlyHints() throws IOException {
-        // NO-OP for AJP
-    }
-
-
     @Override
     protected final int available(boolean doRead) {
         if (endOfStream) {
@@ -1062,12 +1068,10 @@ public class AjpProcessor extends AbstractProcessor {
         if (empty && doRead) {
             try {
                 refillReadBuffer(false);
-            } catch (IOException ioe) {
-                /*
-                 * Probably a timeout. This approach isn't ideal but it works. Returning 1 will indicate that data is
-                 * available which should trigger a read which in turn will trigger another IOException and that one can
-                 * be thrown.
-                 */
+            } catch (IOException timeout) {
+                // Not ideal. This will indicate that data is available
+                // which should trigger a read which in turn will trigger
+                // another IOException and that one can be thrown.
                 return 1;
             }
         }
@@ -1254,6 +1258,12 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
+    @Override
+    protected ServletConnection getServletConnection() {
+        return socketWrapper.getServletConnection("ajp", "");
+    }
+
+
     // ------------------------------------- InputStreamInputBuffer Inner Class
 
     /**
@@ -1303,8 +1313,8 @@ public class AjpProcessor extends AbstractProcessor {
                 // Validate and write response headers
                 try {
                     prepareResponse();
-                } catch (IOException ioe) {
-                    setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
+                } catch (IOException e) {
+                    setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                 }
             }
 
